@@ -40,7 +40,7 @@
 #   include <io.h>
 #   define mktemp _mktemp
 #endif
-#ifdef _MSC_VER
+#if defined(_MSC_VER) || defined(__MINGW32__)
 #   define SIZE_T_FMTSTR "Iu"
 #else
 #   define SIZE_T_FMTSTR "zu"
@@ -80,12 +80,28 @@ Mat_PrintNumber(enum matio_types type, void *data)
             break;
 #ifdef HAVE_MAT_INT64_T
         case MAT_T_INT64:
-            printf("%lld",*(mat_int64_t*)data);
+#if HAVE_INTTYPES_H
+            printf("%" PRIi64,*(mat_int64_t*)data);
+#elif defined(_MSC_VER) && _MSC_VER >= 1200
+            printf("%I64i",*(mat_int64_t*)data);
+#elif defined(HAVE_LONG_LONG_INT)
+            printf("%lld",(long long)(*(mat_int64_t*)data));
+#else
+            printf("%ld",(long)(*(mat_int64_t*)data));
+#endif
             break;
 #endif
 #ifdef HAVE_MAT_UINT64_T
         case MAT_T_UINT64:
-            printf("%llu",*(mat_uint64_t*)data);
+#if HAVE_INTTYPES_H
+            printf("%" PRIu64,*(mat_uint64_t*)data);
+#elif defined(_MSC_VER) && _MSC_VER >= 1200
+            printf("%I64u",*(mat_uint64_t*)data);
+#elif defined(HAVE_UNSIGNED_LONG_LONG_INT)
+            printf("%llu",(unsigned long long)(*(mat_uint64_t*)data));
+#else
+            printf("%lu",(long)(*(mat_uint64_t*)data));
+#endif
             break;
 #endif
         case MAT_T_INT32:
@@ -196,7 +212,7 @@ Mat_Open(const char *matname,int mode)
     } else if ( (mode & 0x01) == MAT_ACC_RDWR ) {
         fp = fopen( matname, "r+b" );
         if ( !fp ) {
-            mat = Mat_CreateVer(matname,NULL,mode&0xfffffffe);
+            mat = Mat_CreateVer(matname,NULL,(enum mat_ft)(mode&0xfffffffe));
             return mat;
         }
     } else {
@@ -204,16 +220,29 @@ Mat_Open(const char *matname,int mode)
         return NULL;
     }
 
-    mat = malloc(sizeof(*mat));
+    mat = (mat_t*)malloc(sizeof(*mat));
     if ( NULL == mat ) {
-        Mat_Critical("Couldn't allocate memory for the MAT file");
         fclose(fp);
+        Mat_Critical("Couldn't allocate memory for the MAT file");
         return NULL;
     }
 
     mat->fp = fp;
-    mat->header        = calloc(128,1);
-    mat->subsys_offset = calloc(8,1);
+    mat->header        = (char*)calloc(128,sizeof(char));
+    if ( NULL == mat->header ) {
+        free(mat);
+        fclose(fp);
+        Mat_Critical("Couldn't allocate memory for the MAT file header");
+        return NULL;
+    }
+    mat->subsys_offset = (char*)calloc(8,sizeof(char));
+    if ( NULL == mat->subsys_offset ) {
+        free(mat->header);
+        free(mat);
+        fclose(fp);
+        Mat_Critical("Couldn't allocate memory for the MAT file subsys offset");
+        return NULL;
+    }
     mat->filename      = NULL;
     mat->byteswap      = 0;
     mat->version       = 0;
@@ -238,8 +267,16 @@ Mat_Open(const char *matname,int mode)
         mat->version = (int)tmp2;
         if ( (mat->version == 0x0100 || mat->version == 0x0200) &&
              -1 != mat->byteswap ) {
-            mat->bof = ftell(mat->fp);
-            mat->next_index    = 0;
+            mat->bof = ftell((FILE*)mat->fp);
+            if ( mat->bof == -1L ) {
+                free(mat->header);
+                free(mat->subsys_offset);
+                free(mat);
+                fclose(fp);
+                Mat_Critical("Couldn't determine file position");
+                return NULL;
+            }
+            mat->next_index = 0;
         } else {
             mat->version = 0;
         }
@@ -248,10 +285,9 @@ Mat_Open(const char *matname,int mode)
     if ( 0 == mat->version ) {
         /* Maybe a V4 MAT file */
         matvar_t *var;
-        if ( NULL != mat->header )
-            free(mat->header);
-        if ( NULL != mat->subsys_offset )
-            free(mat->subsys_offset);
+
+        free(mat->header);
+        free(mat->subsys_offset);
 
         mat->header        = NULL;
         mat->subsys_offset = NULL;
@@ -268,9 +304,9 @@ Mat_Open(const char *matname,int mode)
         if ( NULL == var &&
              bytesread != 0 ) { /* Accept 0 bytes files as a valid V4 file */
             /* Does not seem to be a valid V4 file */
-            Mat_Critical("\"%s\" does not seem to be a valid MAT file",matname);
             Mat_Close(mat);
             mat = NULL;
+            Mat_Critical("\"%s\" does not seem to be a valid MAT file",matname);
         } else {
             Mat_VarFree(var);
             Mat_Rewind(mat);
@@ -284,7 +320,7 @@ Mat_Open(const char *matname,int mode)
     mat->mode = mode;
 
     if ( mat->version == 0x0200 ) {
-        fclose(mat->fp);
+        fclose((FILE*)mat->fp);
 #if defined(MAT73) && MAT73
 
         mat->fp = malloc(sizeof(hid_t));
@@ -295,9 +331,9 @@ Mat_Open(const char *matname,int mode)
             *(hid_t*)mat->fp=H5Fopen(mat->filename,H5F_ACC_RDWR,H5P_DEFAULT);
 
         if ( -1 < *(hid_t*)mat->fp ) {
-            hsize_t num_objs;
-            H5Gget_num_objs(*(hid_t*)mat->fp,&num_objs);
-            mat->num_datasets = num_objs;
+            H5G_info_t group_info;
+            H5Gget_info(*(hid_t*)mat->fp, &group_info);
+            mat->num_datasets = group_info.nlinks;
             mat->refs_id      = -1;
         }
 #else
@@ -333,7 +369,7 @@ Mat_Close( mat_t *mat )
         }
 #endif
         if ( mat->fp )
-            fclose(mat->fp);
+            fclose((FILE*)mat->fp);
         if ( mat->header )
             free(mat->header);
         if ( mat->subsys_offset )
@@ -349,7 +385,7 @@ Mat_Close( mat_t *mat )
  *
  * Gets the filename for the given MAT file
  * @ingroup MAT
- * @param mat Pointer to the MAT file
+ * @param matfp Pointer to the MAT file
  * @return MAT filename
  */
 const char *
@@ -365,15 +401,15 @@ Mat_GetFilename(mat_t *matfp)
  *
  * Gets the version of the given MAT file
  * @ingroup MAT
- * @param mat Pointer to the MAT file
+ * @param matfp Pointer to the MAT file
  * @return MAT file version
  */
 enum mat_ft
 Mat_GetVersion(mat_t *matfp)
 {
-    enum mat_ft file_type = 0;
+    enum mat_ft file_type = MAT_FT_UNDEFINED;
     if ( NULL != matfp )
-        file_type = matfp->version;
+        file_type = (enum mat_ft)matfp->version;
     return file_type;
 }
 
@@ -387,20 +423,23 @@ Mat_GetVersion(mat_t *matfp)
 int
 Mat_Rewind( mat_t *mat )
 {
+    int err = 0;
+
     switch ( mat->version ) {
         case MAT_FT_MAT73:
             mat->next_index = 0;
             break;
         case MAT_FT_MAT5:
-            fseek(mat->fp,128L,SEEK_SET);
+            (void)fseek((FILE*)mat->fp,128L,SEEK_SET);
             break;
         case MAT_FT_MAT4:
-            fseek(mat->fp,0L,SEEK_SET);
+            (void)fseek((FILE*)mat->fp,0L,SEEK_SET);
             break;
         default:
-            return -1;
+            err = -1;
+            break;
     }
-    return 0;
+    return err;
 }
 
 /** @brief Returns the size of a Matlab Class
@@ -461,7 +500,7 @@ Mat_VarCalloc(void)
 {
     matvar_t *matvar;
 
-    matvar = malloc(sizeof(*matvar));
+    matvar = (matvar_t*)malloc(sizeof(*matvar));
 
     if ( NULL != matvar ) {
         matvar->nbytes       = 0;
@@ -476,8 +515,8 @@ Mat_VarCalloc(void)
         matvar->name         = NULL;
         matvar->data         = NULL;
         matvar->mem_conserve = 0;
-        matvar->compression  = 0;
-        matvar->internal     = malloc(sizeof(*matvar->internal));
+        matvar->compression  = MAT_COMPRESSION_NONE;
+        matvar->internal     = (struct matvar_internal*)malloc(sizeof(*matvar->internal));
         if ( NULL == matvar->internal ) {
             free(matvar);
             matvar = NULL;
@@ -486,12 +525,13 @@ Mat_VarCalloc(void)
             matvar->internal->hdf5_ref  =  0;
             matvar->internal->id        = -1;
             matvar->internal->fp = NULL;
-            matvar->internal->fpos         = 0;
-            matvar->internal->datapos      = 0;
-            matvar->internal->fieldnames   = NULL;
-            matvar->internal->num_fields   = 0;
+            matvar->internal->fpos       = 0;
+            matvar->internal->datapos    = 0;
+            matvar->internal->fieldnames = NULL;
+            matvar->internal->num_fields = 0;
 #if defined(HAVE_ZLIB)
-            matvar->internal->z         = NULL;
+            matvar->internal->z          = NULL;
+            matvar->internal->data       = NULL;
 #endif
         }
     }
@@ -575,7 +615,7 @@ Mat_VarCreate(const char *name,enum matio_classes class_type,
     if ( name )
         matvar->name = strdup_printf("%s",name);
     matvar->rank = rank;
-    matvar->dims = malloc(matvar->rank*sizeof(*matvar->dims));
+    matvar->dims = (size_t*)malloc(matvar->rank*sizeof(*matvar->dims));
     for ( i = 0; i < matvar->rank; i++ ) {
         matvar->dims[i] = dims[i];
         nmemb *= dims[i];
@@ -631,7 +671,7 @@ Mat_VarCreate(const char *name,enum matio_classes class_type,
 
             data_size = sizeof(matvar_t **);
             if ( data != NULL ) {
-                fields = data;
+                fields = (matvar_t**)data;
                 nfields = 0;
                 while ( fields[nfields] != NULL )
                     nfields++;
@@ -640,7 +680,7 @@ Mat_VarCreate(const char *name,enum matio_classes class_type,
                 matvar->internal->num_fields = nfields;
                 if ( nfields ) {
                     matvar->internal->fieldnames =
-                        calloc(nfields,sizeof(*matvar->internal->fieldnames));
+                        (char**)calloc(nfields,sizeof(*matvar->internal->fieldnames));
                     for ( i = 0; i < nfields; i++ )
                         matvar->internal->fieldnames[i] = strdup(fields[i]->name);
                     nmemb *= nfields;
@@ -649,8 +689,8 @@ Mat_VarCreate(const char *name,enum matio_classes class_type,
             break;
         }
         default:
-            Mat_Error("Unrecognized data_type");
             Mat_VarFree(matvar);
+            Mat_Critical("Unrecognized data_type");
             return NULL;
     }
     if ( matvar->class_type == MAT_C_SPARSE ) {
@@ -671,18 +711,18 @@ Mat_VarCreate(const char *name,enum matio_classes class_type,
     } else if ( MAT_C_SPARSE == matvar->class_type ) {
         mat_sparse_t *sparse_data, *sparse_data_in;
 
-        sparse_data_in = data;
-        sparse_data    = malloc(sizeof(mat_sparse_t));
+        sparse_data_in = (mat_sparse_t*)data;
+        sparse_data    = (mat_sparse_t*)malloc(sizeof(mat_sparse_t));
         if ( NULL != sparse_data ) {
             sparse_data->nzmax = sparse_data_in->nzmax;
             sparse_data->nir   = sparse_data_in->nir;
             sparse_data->njc   = sparse_data_in->njc;
             sparse_data->ndata = sparse_data_in->ndata;
-            sparse_data->ir = malloc(sparse_data->nir*sizeof(*sparse_data->ir));
+            sparse_data->ir = (int*)malloc(sparse_data->nir*sizeof(*sparse_data->ir));
             if ( NULL != sparse_data->ir )
                 memcpy(sparse_data->ir,sparse_data_in->ir,
                        sparse_data->nir*sizeof(*sparse_data->ir));
-            sparse_data->jc = malloc(sparse_data->njc*sizeof(*sparse_data->jc));
+            sparse_data->jc = (int*)malloc(sparse_data->njc*sizeof(*sparse_data->jc));
             if ( NULL != sparse_data->jc )
                 memcpy(sparse_data->jc,sparse_data_in->jc,
                        sparse_data->njc*sizeof(*sparse_data->jc));
@@ -690,8 +730,8 @@ Mat_VarCreate(const char *name,enum matio_classes class_type,
                 sparse_data->data = malloc(sizeof(mat_complex_split_t));
                 if ( NULL != sparse_data->data ) {
                     mat_complex_split_t *complex_data,*complex_data_in;
-                    complex_data     = sparse_data->data;
-                    complex_data_in  = sparse_data_in->data;
+                    complex_data     = (mat_complex_split_t*)sparse_data->data;
+                    complex_data_in  = (mat_complex_split_t*)sparse_data_in->data;
                     complex_data->Re = malloc(sparse_data->ndata*data_size);
                     complex_data->Im = malloc(sparse_data->ndata*data_size);
                     if ( NULL != complex_data->Re )
@@ -713,8 +753,8 @@ Mat_VarCreate(const char *name,enum matio_classes class_type,
         if ( matvar->isComplex ) {
             matvar->data   = malloc(sizeof(mat_complex_split_t));
             if ( NULL != matvar->data && matvar->nbytes > 0 ) {
-                mat_complex_split_t *complex_data    = matvar->data;
-                mat_complex_split_t *complex_data_in = data;
+                mat_complex_split_t *complex_data    = (mat_complex_split_t*)matvar->data;
+                mat_complex_split_t *complex_data_in = (mat_complex_split_t*)data;
 
                 complex_data->Re = malloc(matvar->nbytes);
                 complex_data->Im = malloc(matvar->nbytes);
@@ -831,7 +871,7 @@ Mat_VarDelete(mat_t *mat, const char *name)
                 }
 #endif
                 if ( mat->fp ) {
-                    fclose(mat->fp);
+                    fclose((FILE*)mat->fp);
                     mat->fp = NULL;
                 }
 
@@ -912,10 +952,11 @@ Mat_VarDuplicate(const matvar_t *in, int opt)
     out->internal->datapos  = in->internal->datapos;
 #if defined(HAVE_ZLIB)
     out->internal->z        = NULL;
+    out->internal->data     = NULL;
 #endif
     out->internal->num_fields = in->internal->num_fields;
     if ( NULL != in->internal->fieldnames && in->internal->num_fields > 0 ) {
-        out->internal->fieldnames = calloc(in->internal->num_fields,
+        out->internal->fieldnames = (char**)calloc(in->internal->num_fields,
                                            sizeof(*in->internal->fieldnames));
         for ( i = 0; i < in->internal->num_fields; i++ ) {
             if ( NULL != in->internal->fieldnames[i] )
@@ -924,15 +965,71 @@ Mat_VarDuplicate(const matvar_t *in, int opt)
         }
     }
 
-    if (in->name != NULL && (NULL != (out->name = malloc(strlen(in->name)+1))))
+    if (in->name != NULL && (NULL != (out->name = (char*)malloc(strlen(in->name)+1))))
         memcpy(out->name,in->name,strlen(in->name)+1);
 
-    out->dims = malloc(in->rank*sizeof(*out->dims));
+    out->dims = (size_t*)malloc(in->rank*sizeof(*out->dims));
     if ( out->dims != NULL )
         memcpy(out->dims,in->dims,in->rank*sizeof(*out->dims));
+
 #if defined(HAVE_ZLIB)
-    if ( (in->internal->z != NULL) && (NULL != (out->internal->z = malloc(sizeof(z_stream)))) )
+    if ( (in->internal->z != NULL) && (NULL != (out->internal->z = (z_streamp)malloc(sizeof(z_stream)))) )
         inflateCopy(out->internal->z,in->internal->z);
+    if ( in->internal->data != NULL ) {
+        if ( in->class_type == MAT_C_SPARSE ) {
+            out->internal->data = malloc(sizeof(mat_sparse_t));
+            if ( out->internal->data != NULL ) {
+                mat_sparse_t *out_sparse = (mat_sparse_t*)out->internal->data;
+                mat_sparse_t *in_sparse  = (mat_sparse_t*)in->internal->data;
+                out_sparse->nzmax = in_sparse->nzmax;
+                out_sparse->nir = in_sparse->nir;
+                out_sparse->ir = (int*)malloc(in_sparse->nir*sizeof(int));
+                if ( out_sparse->ir != NULL )
+                    memcpy(out_sparse->ir, in_sparse->ir, in_sparse->nir*sizeof(int));
+                out_sparse->njc = in_sparse->njc;
+                out_sparse->jc = (int*)malloc(in_sparse->njc*sizeof(int));
+                if ( out_sparse->jc != NULL )
+                    memcpy(out_sparse->jc, in_sparse->jc, in_sparse->njc*sizeof(int));
+                out_sparse->ndata = in_sparse->ndata;
+                if ( out->isComplex && NULL != in_sparse->data ) {
+                    out_sparse->data = malloc(sizeof(mat_complex_split_t));
+                    if ( out_sparse->data != NULL ) {
+                        mat_complex_split_t *out_data = (mat_complex_split_t*)out_sparse->data;
+                        mat_complex_split_t *in_data  = (mat_complex_split_t*)in_sparse->data;
+                        out_data->Re = malloc(
+                            in_sparse->ndata*Mat_SizeOf(in->data_type));
+                        if ( NULL != out_data->Re )
+                            memcpy(out_data->Re,in_data->Re,
+                                in_sparse->ndata*Mat_SizeOf(in->data_type));
+                        out_data->Im = malloc(
+                            in_sparse->ndata*Mat_SizeOf(in->data_type));
+                        if ( NULL != out_data->Im )
+                            memcpy(out_data->Im,in_data->Im,
+                                in_sparse->ndata*Mat_SizeOf(in->data_type));
+                    }
+                } else if ( in_sparse->data != NULL ) {
+                    out_sparse->data = malloc(in_sparse->ndata*Mat_SizeOf(in->data_type));
+                    if ( NULL != out_sparse->data )
+                        memcpy(out_sparse->data, in_sparse->data,
+                            in_sparse->ndata*Mat_SizeOf(in->data_type));
+                }
+            }
+        } else if ( out->isComplex ) {
+            out->internal->data = malloc(sizeof(mat_complex_split_t));
+            if ( out->internal->data != NULL ) {
+                mat_complex_split_t *out_data = (mat_complex_split_t*)out->internal->data;
+                mat_complex_split_t *in_data  = (mat_complex_split_t*)in->internal->data;
+                out_data->Re = malloc(out->nbytes);
+                if ( NULL != out_data->Re )
+                    memcpy(out_data->Re,in_data->Re,out->nbytes);
+                out_data->Im = malloc(out->nbytes);
+                if ( NULL != out_data->Im )
+                    memcpy(out_data->Im,in_data->Im,out->nbytes);
+            }
+        } else if ( NULL != (out->internal->data = malloc(in->nbytes)) ) {
+            memcpy(out->internal->data, in->internal->data, in->nbytes);
+        }
+    }
 #endif
 
     if ( !opt ) {
@@ -963,12 +1060,45 @@ Mat_VarDuplicate(const matvar_t *in, int opt)
                 outcells[i] = Mat_VarDuplicate(incells[i],opt);
             }
         }
+    } else if ( (in->data != NULL) && (in->class_type == MAT_C_SPARSE) ) {
+        out->data = malloc(sizeof(mat_sparse_t));
+        if ( out->data != NULL ) {
+            mat_sparse_t *out_sparse = (mat_sparse_t*)out->data;
+            mat_sparse_t *in_sparse  = (mat_sparse_t*)in->data;
+            out_sparse->nzmax = in_sparse->nzmax;
+            out_sparse->nir = in_sparse->nir;
+            out_sparse->ir = (int*)malloc(in_sparse->nir*sizeof(int));
+            if ( out_sparse->ir != NULL )
+                memcpy(out_sparse->ir, in_sparse->ir, in_sparse->nir*sizeof(int));
+            out_sparse->njc = in_sparse->njc;
+            out_sparse->jc = (int*)malloc(in_sparse->njc*sizeof(int));
+            if ( out_sparse->jc != NULL )
+                memcpy(out_sparse->jc, in_sparse->jc, in_sparse->njc*sizeof(int));
+            out_sparse->ndata = in_sparse->ndata;
+            if ( out->isComplex && NULL != in_sparse->data ) {
+                out_sparse->data = malloc(sizeof(mat_complex_split_t));
+                if ( out_sparse->data != NULL ) {
+                    mat_complex_split_t *out_data = (mat_complex_split_t*)out_sparse->data;
+                    mat_complex_split_t *in_data  = (mat_complex_split_t*)in_sparse->data;
+                    out_data->Re = malloc(in_sparse->ndata*Mat_SizeOf(in->data_type));
+                    if ( NULL != out_data->Re )
+                        memcpy(out_data->Re,in_data->Re,in_sparse->ndata*Mat_SizeOf(in->data_type));
+                    out_data->Im = malloc(in_sparse->ndata*Mat_SizeOf(in->data_type));
+                    if ( NULL != out_data->Im )
+                        memcpy(out_data->Im,in_data->Im,in_sparse->ndata*Mat_SizeOf(in->data_type));
+                }
+            } else if ( in_sparse->data != NULL ) {
+                out_sparse->data = malloc(in_sparse->ndata*Mat_SizeOf(in->data_type));
+                if ( NULL != out_sparse->data )
+                    memcpy(out_sparse->data, in_sparse->data, in_sparse->ndata*Mat_SizeOf(in->data_type));
+            }
+        }
     } else if ( in->data != NULL ) {
         if ( out->isComplex ) {
             out->data = malloc(sizeof(mat_complex_split_t));
             if ( out->data != NULL ) {
-                mat_complex_split_t *out_data = out->data;
-                mat_complex_split_t *in_data  = in->data;
+                mat_complex_split_t *out_data = (mat_complex_split_t*)out->data;
+                mat_complex_split_t *in_data  = (mat_complex_split_t*)in->data;
                 out_data->Re = malloc(out->nbytes);
                 if ( NULL != out_data->Re )
                     memcpy(out_data->Re,in_data->Re,out->nbytes);
@@ -1010,7 +1140,7 @@ Mat_VarFree(matvar_t *matvar)
         switch (matvar->class_type ) {
             case MAT_C_STRUCT:
                 if ( !matvar->mem_conserve ) {
-                    matvar_t **fields = matvar->data;
+                    matvar_t **fields = (matvar_t**)matvar->data;
                     int nfields = matvar->internal->num_fields;
                     for ( i = 0; i < nmemb*nfields; i++ )
                         Mat_VarFree(fields[i]);
@@ -1020,7 +1150,7 @@ Mat_VarFree(matvar_t *matvar)
                 }
             case MAT_C_CELL:
                 if ( !matvar->mem_conserve ) {
-                    matvar_t **cells = matvar->data;
+                    matvar_t **cells = (matvar_t**)matvar->data;
                     for ( i = 0; i < nmemb; i++ )
                         Mat_VarFree(cells[i]);
 
@@ -1030,13 +1160,13 @@ Mat_VarFree(matvar_t *matvar)
             case MAT_C_SPARSE:
                 if ( !matvar->mem_conserve ) {
                     mat_sparse_t *sparse;
-                    sparse = matvar->data;
+                    sparse = (mat_sparse_t*)matvar->data;
                     if ( sparse->ir != NULL )
                         free(sparse->ir);
                     if ( sparse->jc != NULL )
                         free(sparse->jc);
                     if ( matvar->isComplex && NULL != sparse->data ) {
-                        mat_complex_split_t *complex_data = sparse->data;
+                        mat_complex_split_t *complex_data = (mat_complex_split_t*)sparse->data;
                         free(complex_data->Re);
                         free(complex_data->Im);
                         free(complex_data);
@@ -1059,7 +1189,7 @@ Mat_VarFree(matvar_t *matvar)
             case MAT_C_CHAR:
                 if ( !matvar->mem_conserve ) {
                     if ( matvar->isComplex ) {
-                        mat_complex_split_t *complex_data = matvar->data;
+                        mat_complex_split_t *complex_data = (mat_complex_split_t*)matvar->data;
                         free(complex_data->Re);
                         free(complex_data->Im);
                         free(complex_data);
@@ -1080,6 +1210,32 @@ Mat_VarFree(matvar_t *matvar)
         if ( matvar->compression == MAT_COMPRESSION_ZLIB ) {
             inflateEnd(matvar->internal->z);
             free(matvar->internal->z);
+            if ( (matvar->internal->data != NULL) && (matvar->class_type == MAT_C_SPARSE) ) {
+                mat_sparse_t *sparse;
+                sparse = (mat_sparse_t*)matvar->internal->data;
+                if ( sparse->ir != NULL )
+                    free(sparse->ir);
+                if ( sparse->jc != NULL )
+                    free(sparse->jc);
+                if ( matvar->isComplex && NULL != sparse->data ) {
+                    mat_complex_split_t *complex_data = (mat_complex_split_t*)sparse->data;
+                    free(complex_data->Re);
+                    free(complex_data->Im);
+                    free(complex_data);
+                } else if ( sparse->data != NULL ) {
+                    free(sparse->data);
+                }
+                free(sparse);
+            }
+            else if ( (matvar->internal->data != NULL) && matvar->isComplex ) {
+                mat_complex_split_t *complex_data =
+                    (mat_complex_split_t*)matvar->internal->data;
+                free(complex_data->Re);
+                free(complex_data->Im);
+                free(complex_data);
+            } else if ( NULL != matvar->internal->data ) {
+                free(matvar->internal->data);
+            }
         }
 #endif
 #if defined(MAT73) && MAT73
@@ -1135,54 +1291,12 @@ Mat_VarFree(matvar_t *matvar)
     free(matvar);
 }
 
-void
-Mat_VarFree2(matvar_t *matvar)
-{
-    if ( !matvar )
-        return;
-    if ( matvar->dims )
-        free(matvar->dims);
-    if ( matvar->name )
-        free(matvar->name);
-    if ( (matvar->data != NULL) && (matvar->class_type == MAT_C_STRUCT ||
-          matvar->class_type == MAT_C_CELL) && matvar->data_size > 0 ) {
-        int i;
-        matvar_t **fields = (matvar_t **)matvar->data;
-        int nfields = matvar->nbytes / matvar->data_size;
-        for ( i = 0; i < nfields; i++ )
-            Mat_VarFree(fields[i]);
-        free(matvar->data);
-    } else if ( (matvar->data != NULL) && (!matvar->mem_conserve) &&
-                (matvar->class_type == MAT_C_SPARSE) ) {
-        mat_sparse_t *sparse;
-        sparse = matvar->data;
-        if ( sparse->ir != NULL )
-            free(sparse->ir);
-        if ( sparse->jc != NULL )
-            free(sparse->jc);
-        if ( sparse->data != NULL )
-            free(sparse->data);
-        free(sparse);
-    } else {
-        if ( matvar->data && !matvar->mem_conserve )
-            free(matvar->data);
-    }
-#if defined(HAVE_ZLIB)
-    if ( matvar->compression == MAT_COMPRESSION_ZLIB )
-        inflateEnd(matvar->internal->z);
-#endif
-    /* FIXME: Why does this cause a SEGV? */
-#if 0
-    memset(matvar,0,sizeof(matvar_t));
-#endif
-}
-
 /** @brief Calculate a single subscript from a set of subscript values
  *
  * Calculates a single linear subscript (0-relative) given a 1-relative
  * subscript for each dimension.  The calculation uses the formula below where
  * index is the linear index, s is an array of length RANK where each element
- * is the subscript for the correspondind dimension, D is an array whose
+ * is the subscript for the corresponding dimension, D is an array whose
  * elements are the dimensions of the variable.
  * \f[
  *   index = \sum\limits_{k=0}^{RANK-1} [(s_k - 1) \prod\limits_{l=0}^{k} D_l ]
@@ -1224,7 +1338,7 @@ Mat_CalcSingleSubscript(int rank,int *dims,int *subs)
  * Calculates a single linear subscript (0-relative) given a 1-relative
  * subscript for each dimension.  The calculation uses the formula below where
  * index is the linear index, s is an array of length RANK where each element
- * is the subscript for the correspondind dimension, D is an array whose
+ * is the subscript for the corresponding dimension, D is an array whose
  * elements are the dimensions of the variable.
  * \f[
  *   index = \sum\limits_{k=0}^{RANK-1} [(s_k - 1) \prod\limits_{l=0}^{k} D_l ]
@@ -1285,7 +1399,7 @@ Mat_CalcSubscripts(int rank,int *dims,int index)
     int i, j, k, *subs;
     double l;
 
-    subs = malloc(rank*sizeof(int));
+    subs = (int*)malloc(rank*sizeof(int));
     l = index;
     for ( i = rank; i--; ) {
         k = 1;
@@ -1324,7 +1438,7 @@ Mat_CalcSubscripts2(int rank,size_t *dims,size_t index)
     size_t *subs;
     double l;
 
-    subs = malloc(rank*sizeof(size_t));
+    subs = (size_t*)malloc(rank*sizeof(size_t));
     l = (double)index;
     for ( i = rank; i--; ) {
         int j;
@@ -1354,21 +1468,26 @@ Mat_VarGetSize(matvar_t *matvar)
     if ( matvar->class_type == MAT_C_STRUCT ) {
         int nfields;
         matvar_t **fields;
-        /* This is really nmemb*nfields, but we'll get a
-         * more accurate count of the bytes by loopoing over all of them
-         */
+
+        nmemb = 1;
+        for ( i = 0; i < matvar->rank; i++ )
+            nmemb *= matvar->dims[i];
         nfields = matvar->internal->num_fields;
-        fields  = matvar->data;
-        for ( i = 0; i < nfields; i++ )
-            bytes += Mat_VarGetSize(fields[i]);
+        if ( nmemb*nfields > 0 ) {
+            fields  = (matvar_t**)matvar->data;
+            if ( NULL != fields )
+                for ( i = 0; i < nmemb*nfields; i++ )
+                    bytes += Mat_VarGetSize(fields[i]);
+        }
     } else if ( matvar->class_type == MAT_C_CELL ) {
         int ncells;
         matvar_t **cells;
 
         ncells = matvar->nbytes / matvar->data_size;
-        cells  = matvar->data;
-        for ( i = 0; i < ncells; i++ )
-            bytes += Mat_VarGetSize(cells[i]);
+        cells  = (matvar_t**)matvar->data;
+        if ( NULL != cells )
+            for ( i = 0; i < ncells; i++ )
+                bytes += Mat_VarGetSize(cells[i]);
     } else {
         nmemb = 1;
         for ( i = 0; i < matvar->rank; i++ )
@@ -1492,10 +1611,10 @@ Mat_VarPrint( matvar_t *matvar, int printdata )
             {
                 size_t stride = Mat_SizeOf(matvar->data_type);
                 if ( matvar->isComplex ) {
-                    mat_complex_split_t *complex_data = matvar->data;
-                    char *rp = complex_data->Re;
-                    char *ip = complex_data->Im;
-                   for ( i = 0; i < matvar->dims[0] && i < 15; i++ ) {
+                    mat_complex_split_t *complex_data = (mat_complex_split_t*)matvar->data;
+                    char *rp = (char*)complex_data->Re;
+                    char *ip = (char*)complex_data->Im;
+                    for ( i = 0; i < matvar->dims[0] && i < 15; i++ ) {
                         for ( j = 0; j < matvar->dims[1] && j < 15; j++ ) {
                             size_t idx = matvar->dims[0]*j+i;
                             Mat_PrintNumber(matvar->data_type,rp+idx*stride);
@@ -1510,7 +1629,7 @@ Mat_VarPrint( matvar_t *matvar, int printdata )
                     if ( i < matvar->dims[0] )
                         printf(".\n.\n.\n");
                } else {
-                   char *data = matvar->data;
+                   char *data = (char*)matvar->data;
                    for ( i = 0; i < matvar->dims[0] && i < 15; i++ ) {
                         for ( j = 0; j < matvar->dims[1] && j < 15; j++ ) {
                             size_t idx = matvar->dims[0]*j+i;
@@ -1529,9 +1648,7 @@ Mat_VarPrint( matvar_t *matvar, int printdata )
             }
             case MAT_C_CHAR:
             {
-                char *data = matvar->data;
-                if ( !printdata )
-                    break;
+                char *data = (char*)matvar->data;
                 for ( i = 0; i < matvar->dims[0]; i++ ) {
                     for ( j = 0; j < matvar->dims[1]; j++ )
                         printf("%c",data[j*matvar->dims[0]+i]);
@@ -1547,15 +1664,14 @@ Mat_VarPrint( matvar_t *matvar, int printdata )
                 if ( MAT_T_DOUBLE != matvar->data_type )
                     break;
 #endif
-                sparse = matvar->data;
+                sparse = (mat_sparse_t*)matvar->data;
                 if ( matvar->isComplex ) {
-                    mat_complex_split_t *complex_data = sparse->data;
-                    char *re,*im;
-                    re = complex_data->Re;
-                    im = complex_data->Im;
+                    mat_complex_split_t *complex_data = (mat_complex_split_t*)sparse->data;
+                    char *re = (char*)complex_data->Re;
+                    char *im = (char*)complex_data->Im;
                     for ( i = 0; i < sparse->njc-1; i++ ) {
-                        for (j = sparse->jc[i];
-                             j<sparse->jc[i+1] && j<sparse->ndata;j++ ) {
+                        for ( j = sparse->jc[i];
+                              j < sparse->jc[i+1] && j < sparse->ndata; j++ ) {
                             printf("    (%d,%d)  ",sparse->ir[j]+1,i+1);
                             Mat_PrintNumber(matvar->data_type,re+j*stride);
                             printf(" + ");
@@ -1564,11 +1680,10 @@ Mat_VarPrint( matvar_t *matvar, int printdata )
                         }
                     }
                 } else {
-                    char *data;
-                    data = sparse->data;
+                    char *data = (char*)sparse->data;
                     for ( i = 0; i < sparse->njc-1; i++ ) {
-                        for (j = sparse->jc[i];
-                             j<sparse->jc[i+1] && j<sparse->ndata;j++ ){
+                        for ( j = sparse->jc[i];
+                              j < sparse->jc[i+1] && j < sparse->ndata; j++ ) {
                             printf("    (%d,%d)  ",sparse->ir[j]+1,i+1);
                             Mat_PrintNumber(matvar->data_type,data+j*stride);
                             printf("\n");
@@ -1764,8 +1879,7 @@ Mat_VarReadNextInfo( mat_t *mat )
 matvar_t *
 Mat_VarReadInfo( mat_t *mat, const char *name )
 {
-
-    long  fpos;
+    long fpos;
     matvar_t *matvar = NULL;
 
     if ( (mat == NULL) || (name == NULL) )
@@ -1777,10 +1891,7 @@ Mat_VarReadInfo( mat_t *mat, const char *name )
         do {
             matvar = Mat_VarReadNextInfo(mat);
             if ( matvar != NULL ) {
-                if ( !matvar->name ) {
-                    Mat_VarFree(matvar);
-                    matvar = NULL;
-                } else if ( strcmp(matvar->name,name) ) {
+                if ( matvar->name == NULL || strcmp(matvar->name,name) ) {
                     Mat_VarFree(matvar);
                     matvar = NULL;
                 }
@@ -1788,28 +1899,28 @@ Mat_VarReadInfo( mat_t *mat, const char *name )
                 Mat_Critical("An error occurred in reading the MAT file");
                 break;
             }
-        } while ( NULL == matvar && mat->next_index < mat->num_datasets);
+        } while ( NULL == matvar && mat->next_index < mat->num_datasets );
         mat->next_index = fpos;
     } else {
-        fpos = ftell(mat->fp);
-        fseek(mat->fp,mat->bof,SEEK_SET);
-        do {
-            matvar = Mat_VarReadNextInfo(mat);
-            if ( matvar != NULL ) {
-                if ( !matvar->name ) {
-                    Mat_VarFree(matvar);
-                    matvar = NULL;
-                } else if ( strcmp(matvar->name,name) ) {
-                    Mat_VarFree(matvar);
-                    matvar = NULL;
+        fpos = ftell((FILE*)mat->fp);
+        if ( fpos != -1L ) {
+            (void)fseek((FILE*)mat->fp,mat->bof,SEEK_SET);
+            do {
+                matvar = Mat_VarReadNextInfo(mat);
+                if ( matvar != NULL ) {
+                    if ( matvar->name == NULL || strcmp(matvar->name,name) ) {
+                        Mat_VarFree(matvar);
+                        matvar = NULL;
+                    }
+                } else if (!feof((FILE *)mat->fp)) {
+                    Mat_Critical("An error occurred in reading the MAT file");
+                    break;
                 }
-            } else {
-                Mat_Critical("An error occurred in reading the MAT file");
-                break;
-            }
-        } while ( !matvar && !feof(((FILE *)mat->fp)) );
-
-        fseek(mat->fp,fpos,SEEK_SET);
+            } while ( NULL == matvar && !feof((FILE *)mat->fp) );
+            (void)fseek((FILE*)mat->fp,fpos,SEEK_SET);
+        } else {
+            Mat_Critical("Couldn't determine file position");
+        }
     }
     return matvar;
 }
@@ -1826,15 +1937,19 @@ Mat_VarReadInfo( mat_t *mat, const char *name )
 matvar_t *
 Mat_VarRead( mat_t *mat, const char *name )
 {
-    long  fpos = 0;
-    matvar_t *matvar = NULL;;
+    long fpos;
+    matvar_t *matvar = NULL;
 
     if ( (mat == NULL) || (name == NULL) )
         return NULL;
 
-    if ( MAT_FT_MAT73 != mat->version )
-        fpos = ftell(mat->fp);
-    else {
+    if ( MAT_FT_MAT73 != mat->version ) {
+        fpos = ftell((FILE*)mat->fp);
+        if ( fpos == -1L ) {
+            Mat_Critical("Couldn't determine file position");
+            return NULL;
+        }
+    } else {
         fpos = mat->next_index;
         mat->next_index = 0;
     }
@@ -1843,9 +1958,9 @@ Mat_VarRead( mat_t *mat, const char *name )
     if ( matvar )
         ReadData(mat,matvar);
 
-    if ( MAT_FT_MAT73 != mat->version )
-        fseek(mat->fp,fpos,SEEK_SET);
-    else {
+    if ( MAT_FT_MAT73 != mat->version ) {
+        (void)fseek((FILE*)mat->fp,fpos,SEEK_SET);
+    } else {
         mat->next_index = fpos;
     }
     return matvar;
@@ -1866,16 +1981,22 @@ Mat_VarReadNext( mat_t *mat )
     matvar_t *matvar = NULL;
 
     if ( mat->version != MAT_FT_MAT73 ) {
-        if ( feof(((FILE *)mat->fp)) )
+        if ( feof((FILE *)mat->fp) )
             return NULL;
         /* Read position so we can reset the file position if an error occurs */
-        fpos = ftell(mat->fp);
+        fpos = ftell((FILE*)mat->fp);
+        if ( fpos == -1L ) {
+            Mat_Critical("Couldn't determine file position");
+            return NULL;
+        }
     }
     matvar = Mat_VarReadNextInfo(mat);
-    if ( matvar )
+    if ( matvar ) {
         ReadData(mat,matvar);
-    else if (mat->version != MAT_FT_MAT73 )
-        fseek(mat->fp,fpos,SEEK_SET);
+    } else if (mat->version != MAT_FT_MAT73 ) {
+        (void)fseek((FILE*)mat->fp,fpos,SEEK_SET);
+    }
+
     return matvar;
 }
 
@@ -1891,16 +2012,16 @@ Mat_VarReadNext( mat_t *mat )
 int
 Mat_VarWriteInfo(mat_t *mat, matvar_t *matvar )
 {
+    int err = 0;
+
     if ( mat == NULL || matvar == NULL || mat->fp == NULL )
         return -1;
-    else if ( mat->version != MAT_FT_MAT4 )
+    else if ( mat->version == MAT_FT_MAT5 )
         WriteInfo5(mat,matvar);
-#if 0
-    else if ( mat->version == MAT_FT_MAT4 )
-        WriteInfo4(mat,matvar);
-#endif
+    else
+        err = 1;
 
-    return 0;
+    return err;
 }
 
 /** @brief Writes the given data to the MAT variable
@@ -1922,9 +2043,12 @@ Mat_VarWriteData(mat_t *mat,matvar_t *matvar,void *data,
 {
     int err = 0, k, N = 1;
 
-    fseek(mat->fp,matvar->internal->datapos+8,SEEK_SET);
+    if ( mat == NULL || matvar == NULL )
+        return -1;
 
-    if ( mat == NULL || matvar == NULL || data == NULL ) {
+    (void)fseek((FILE*)mat->fp,matvar->internal->datapos+8,SEEK_SET);
+
+    if ( data == NULL ) {
         err = -1;
     } else if ( start == NULL && stride == NULL && edge == NULL ) {
         for ( k = 0; k < matvar->rank; k++ )
